package dados.produto;


import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;

import util.IteratorBidirecional;

import negocio.exception.CopiaExistenteException;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.LinkFotoExistenteException;
import negocio.exception.LinkInexistenteException;
import negocio.produto.GerenciadorLinkFotos;


import dados.exception.FilialInvalidaException;
import dados.exception.ProdutoInvalidoException;
import dados.filial.Filial;
import dados.locacao.Copia;

@Entity
@Table(name="Produto")
@Inheritance(strategy=InheritanceType.JOINED)
@DiscriminatorColumn(
		name="TipoProduto",
		discriminatorType=DiscriminatorType.STRING
)
@DiscriminatorValue("ProdutoAbstrato")
public abstract class Produto implements Serializable{

	private static final long serialVersionUID = -4823912301205949361L;

	@Id
	@Column(name="Nome")
	protected String nome;

	@Column(name="id")
	protected long id;

	@Column(updatable=true, name="AnoLancamento", nullable=false)
	protected Integer anoLancamento;

	@Column(updatable=true, name="Genero", nullable=false)
	protected String genero;

	@Column(updatable=true, name="Censura", nullable=false)
	protected String censura;

	@Column(updatable=true, name="Descricao", nullable=false, length=2000)
	protected String descricao;

	@Column(updatable=true, name="EmpresaProdutora", nullable=false)
	protected String empresaProdutora;

	@Column(updatable=true, name="linkTrailer", nullable=false)
	protected String linkTrailer;

	@ManyToOne(cascade={CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, optional=false, fetch=FetchType.EAGER)
	@JoinColumn(name="id_Classificacao")
	protected ClassificacaoPreco classificacaoPreco;

	@Embedded
	protected GerenciadorLinkFotos linkFotos;
	
	@Column(name="Nota")
	protected int nota;

	@OneToMany(cascade = CascadeType.ALL, fetch=FetchType.EAGER)
	protected Set<Copia> copias;

	public Produto() {}

	public Produto(String nome, String genero, String censura, String descricao,
			String empresaProdutora, String linkTrailer, ClassificacaoPreco classificacao,
			Integer ano) throws ProdutoInvalidoException {
		if(Produto.isValido(nome, genero, censura, descricao, empresaProdutora, linkTrailer, classificacao, ano)){
			this.nome = nome.trim();
			this.genero = genero.trim();
			this.censura = censura.trim();
			this.descricao = descricao.trim();
			this.empresaProdutora = empresaProdutora.trim();
			this.linkTrailer = linkTrailer.trim();
			this.classificacaoPreco = classificacao;
			this.linkFotos = new GerenciadorLinkFotos();
			this.anoLancamento = ano;
			this.copias = new HashSet<Copia>();
			this.nota = 0;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public static boolean isValido(String nome, String genero, String censura, String descricao,
			String empresaProdutora, String linkTrailer, ClassificacaoPreco classificacao, Integer ano){
		return nome != null && genero != null && censura != null && descricao != null
		&& empresaProdutora != null && linkTrailer != null && classificacao != null
		&& ano > 0;
	}

	public String getNome() {
		return this.nome;
	}

	public void setNome(String nome) throws ProdutoInvalidoException {
		if(Produto.isValido(nome, this.genero, this.censura, this.descricao, this.empresaProdutora, this.linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.nome = nome;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getGenero() {
		return this.genero;
	}

	public void setGenero(String genero) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, genero, this.censura, this.descricao, this.empresaProdutora, this.linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.genero = genero;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getCensura() {
		return this.censura;
	}

	public void setCensura(String censura) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, censura, this.descricao, this.empresaProdutora, this.linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.censura = censura;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getDescricao() {
		return this.descricao;
	}

	public void setDescricao(String descricao) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, this.censura, descricao, this.empresaProdutora, this.linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.descricao = descricao;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getEmpresaProdutora() {
		return this.empresaProdutora;
	}

	public void setEmpresaProdutora(String empresaProdutora) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, this.censura, this.descricao, empresaProdutora, this.linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.empresaProdutora = empresaProdutora;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public String getLinkTrailer() {
		return this.linkTrailer;
	}

	public void setLinkTrailer(String linkTrailer) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, this.censura, this.descricao, this.empresaProdutora, linkTrailer, this.classificacaoPreco, this.anoLancamento)){
			this.linkTrailer = linkTrailer;
		}else{
			throw new ProdutoInvalidoException();
		}
	}
	
	public IteratorBidirecional<LinkFoto> getLinks(){
		return this.linkFotos.getLinks();
	}

	public IteratorBidirecional<LinkFoto> getLinkFotos() {
		IteratorBidirecional<LinkFoto> it = this.linkFotos.getLinks();
		LinkFoto link;
		
		while(it.hasNext()){
			link = it.next();
			if(link.getTipoLinkFoto() == TipoLinkFoto.FOTOBOX || link.getTipoLinkFoto() == TipoLinkFoto.POSTER){
				it.remove();
			}
		}
		
		while(it.hasPrevious()){
			it.previous();
		}
		
		return it;
	}
	
	public LinkFoto getLinkPoster(){
		return this.linkFotos.getLinkPoster();
	}
	
	public LinkFoto getLinkBox(){
		return this.linkFotos.getLinkBox();
	}

	public void inserirLinkFoto(LinkFoto f) throws LinkFotoExistenteException{
		if(f != null){
			this.linkFotos.inserirLink(f);
		}else{
			throw new IllegalArgumentException();
		}
	}

	public void removerLinkFoto(String link) throws LinkInexistenteException{
		if(link != null){
			this.linkFotos.removerLink(link);
		}else{
			throw new IllegalArgumentException();
		}
	}

	public ClassificacaoPreco getClassificacaoPreco() {
		return this.classificacaoPreco;
	}

	public void setClassificacaoPreco(ClassificacaoPreco classificacaoPreco) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, this.censura, this.descricao, this.empresaProdutora, this.linkTrailer, classificacaoPreco, this.anoLancamento)){
			this.classificacaoPreco = classificacaoPreco;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public int getAnoLancamento() {
		return this.anoLancamento;
	}

	public void setAnoLancamento(int anoLancamento) throws ProdutoInvalidoException {
		if(Produto.isValido(this.nome, this.genero, this.censura, this.descricao, this.empresaProdutora, this.linkTrailer, this.classificacaoPreco, anoLancamento)){
			this.anoLancamento = anoLancamento;
		}else{
			throw new ProdutoInvalidoException();
		}
	}

	public void inserirCopia(Copia copia) throws CopiaExistenteException{
		if(copia != null){
			if(copia != null){
				if(!this.copias.contains(copia)){
					this.copias.add(copia);
				}else{
					throw new CopiaExistenteException(copia);
				}
			}else{
				throw new IllegalArgumentException();
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	public void removerCopia(int codigoCopia) throws CopiaInexistenteException{
		IteratorBidirecional<Copia> it = this.getCopias();
		Copia c = null;

		while(c == null && it.hasNext()){
			c = it.next();

			if(c.getCodigoCopia() != codigoCopia){
				c = null;
			}
		}

		if(c != null){
			this.copias.remove(c);
		}else{
			throw new CopiaInexistenteException(codigoCopia);
		}
	}

	public Copia getCopia(int codigo) throws CopiaInexistenteException{
		IteratorBidirecional<Copia> it = this.getCopias();
		Copia copia = null;
		boolean achou = false;

		while(it.hasNext() && !achou){
			copia = it.next();
			if(copia.getCodigoCopia() == codigo){
				achou = true;
			}
		}

		if(achou){
			return copia;
		} else {
			throw new CopiaInexistenteException(codigo);
		}
	}

	public void mudarFilialCopia(int codigoCopia, Filial filial) throws CopiaInexistenteException, FilialInvalidaException{
		Copia atual = this.getCopia(codigoCopia);
		atual.setFilial(filial);
	}

	public IteratorBidirecional<Copia> getCopias() throws CopiaInexistenteException{
		if(!this.copias.isEmpty()){
			return new IteratorBidirecional<Copia>(this.copias.iterator());
		}else{
			throw new CopiaInexistenteException();
		}
	}

	@Override
	public boolean equals(Object arg0) {
		return arg0 != null && arg0 instanceof Produto &&
		this.nome == ((Produto)arg0).getNome();
	}

	@Override
	public String toString() {
		return this.nome;
	}
	
	public GerenciadorLinkFotos getGerenciador(){
		return this.linkFotos;
	}

	public abstract void setDados(Produto dados) throws ProdutoInvalidoException;

	public int getNota() {
		return this.nota;
	}

	public void setNota(int nota) {
		this.nota = nota;
	}

	public long getId() {
		return id;
	}

	public void setId(long id) {
		this.id = id;
	}

}
