package dados.usuario;


import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToOne;

import org.hibernate.annotations.Fetch;
import org.hibernate.annotations.FetchMode;

import util.IteratorBidirecional;

import negocio.GerenciadorTelefone;
import negocio.exception.SenhaInvalidaException;

import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.EnderecoInvalidoException;
import dados.exception.NomeInvalidoException;
import dados.exception.PlanoInvalidoException;
import dados.filial.Endereco;

@Entity
@DiscriminatorValue("Cliente")
public class Cliente extends Usuario {

	private static final long serialVersionUID = 6299930549049873248L;

	@ManyToOne(cascade = {CascadeType.MERGE, CascadeType.PERSIST, CascadeType.REFRESH}, optional=false, targetEntity=Plano.class, fetch=FetchType.EAGER)
	@Fetch(FetchMode.JOIN)
	@JoinColumn(name="id_Plano")
	private Plano plano;

	@OneToOne(cascade = CascadeType.ALL, optional=false, fetch=FetchType.EAGER)
	@JoinColumn(name="id_Endereco")
	private Endereco endereco;

	@Column(updatable=true, name="LocaesGratuitas", nullable=false)
	private Integer locacoesGratuitas;

	@Embedded
	private GerenciadorTelefone telefones;

	public Cliente() {}

	public Cliente(long cpf, String senha, String nome, String email, Plano plano, Endereco endereco) throws NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, EnderecoInvalidoException, PlanoInvalidoException, EmailInvalidoException {
		super(cpf, senha, nome, email);

		if(plano != null){
			if(endereco != null){
				this.plano = plano;
				this.endereco = endereco;
				this.locacoesGratuitas = this.plano.getLocacoesGratuitas();
				this.telefones = new GerenciadorTelefone();
			} else {
				throw new EnderecoInvalidoException();
			}
		} else {
			throw new PlanoInvalidoException();
		}
	}

	private static boolean isValido(Plano plano, Endereco endereco) {
		return plano != null && endereco != null;
	}

	public Plano getPlano() {
		return this.plano;
	}

	public void setPlano(Plano plano) throws PlanoInvalidoException {
		if(Cliente.isValido(plano, this.endereco)){
			this.plano = plano;
		}else{
			throw new PlanoInvalidoException();
		}
	}

	public Endereco getEndereco() {
		return this.endereco;
	}

	public void setEndereco(Endereco endereco) throws EnderecoInvalidoException {
		if(Cliente.isValido(this.plano, endereco)){
			this.endereco = endereco;
		}else{
			throw new EnderecoInvalidoException();
		}
	}

	public IteratorBidirecional<Telefone> getTelefones() {
		return this.telefones.getTelefones();
	}

	public void decrementarLocacoesGratuitas() {
		if(this.locacoesGratuitas > 0){
			this.locacoesGratuitas = this.locacoesGratuitas - 1;
		}else{
			throw new IllegalStateException();
		}
	}

	public void incrementarLocacoesGratuitas() {
		this.locacoesGratuitas = this.locacoesGratuitas + 1;
	}

	public boolean podeLocarGratuitamente(){
		return this.locacoesGratuitas > 0;
	}

	public int getLocacoesGratuitas() {
		return this.locacoesGratuitas;
	}

	@Override
	public boolean temPermissaoCadastro(Usuario user) {
		if(user != null){
			return user instanceof Dependente && this.equals(((Dependente)user).getCliente());
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoRemocao(Usuario user) {
		if(user != null){
			return user.equals(this) || (user instanceof Dependente && this.equals(((Dependente) user).getCliente()));
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public void setDados(Usuario usuario) {
		if(usuario != null && usuario instanceof Cliente){
			Cliente c = (Cliente) usuario;

			try {
				this.setSenha(c.getSenha());
				this.setEmail(c.getEmail());
				this.setNome(c.getNome());
				this.setPlano(c.getPlano());
				this.setEndereco(c.getEndereco());
				this.setTelefones(c.getTelefones());
			} catch (SenhaInvalidaException e) {
				// NAO deve ocorrer
			} catch (EmailInvalidoException e) {
				// NAO deve ocorrer
			} catch (PlanoInvalidoException e) {
				// NAO deve ocorrer
			} catch (EnderecoInvalidoException e) {
				// NAO deve ocorrer
			} catch (NomeInvalidoException e) {
				// NAO deve ocorrer
			}
		}else{
			throw new IllegalArgumentException();
		}
	}

	@Override
	public boolean temPermissaoAlteracao(Usuario user) {
		if(user != null){
			return user.equals(this);
		}else{
			throw new IllegalArgumentException();
		}
	}

	public void setTelefones(Telefone[] telefones2) {
		this.telefones.setTelefones(telefones2);
	}
	
	public void setTelefones(IteratorBidirecional<Telefone> t){
		this.telefones.limpar();
		while(t.hasNext()){
			this.telefones.inserirTelefone(t.next());
		}
	}

}
