package dados.usuario;

import java.io.Serializable;

import javax.persistence.Column;
import javax.persistence.DiscriminatorColumn;
import javax.persistence.DiscriminatorType;
import javax.persistence.DiscriminatorValue;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Inheritance;
import javax.persistence.InheritanceType;
import javax.persistence.Table;

import negocio.exception.SenhaInvalidaException;

import dados.exception.CpfInvalidoException;
import dados.exception.EmailInvalidoException;
import dados.exception.NomeInvalidoException;

@Entity
@Table(name="Usuario")
@Inheritance(strategy=InheritanceType.JOINED)
@DiscriminatorColumn(
		name="TipoUsuario",
		discriminatorType=DiscriminatorType.STRING
)
@DiscriminatorValue("UsuarioAbstrato")
public abstract class Usuario implements Serializable {

	private static final long serialVersionUID = 857705655100949677L;

	@Id
	@Column(name="id_Usuario")
	private Long cpf;

	@Column(updatable=true, name="Senha", nullable=false)
	private String senha;

	@Column(updatable=true, name="Nome", nullable=false)
	private String nome;

	@Column(updatable=true, name="Email", nullable=false)
	private String email;

	public Usuario(long cpf, String senha, String nome, String email) throws NomeInvalidoException, SenhaInvalidaException, CpfInvalidoException, EmailInvalidoException{
		if(Usuario.validarCPF(cpf)){
			if(Usuario.senhaValida(senha)){
				if(Usuario.nomeValido(nome)){
					if(Usuario.emailValido(email)){
						this.cpf = cpf;
						this.senha = senha;
						this.nome = nome;
						this.email = email;
					} else {
						throw new EmailInvalidoException();
					}
				} else {
					throw new NomeInvalidoException();
				}
			} else {
				throw new SenhaInvalidaException();
			}
		}else{
			throw new CpfInvalidoException();
		}
	}

	public Usuario() {
	}

	public abstract void setDados(Usuario usuario);

	public static boolean nomeValido(String nome){
		return nome != null && !nome.isEmpty() && nome.length() <= 100;
	}

	public static boolean emailValido(String email){
		return email != null && !email.isEmpty() && email.contains("@") && email.length() <= 70;
	}

	private static boolean senhaValida(String senha) {
		boolean retorno = senha != null && !senha.isEmpty() && senha.length() > 5 && senha.length() <= 20;

		if(retorno){
			int i = 0, size = senha.length();

			while(i < size){
				if(Character.isWhitespace(senha.charAt(i))){
					i = size + 1;
				}else{
					i++;
				}
			}

			retorno = (i != size + 1);
		}

		return retorno;
	}

	private static boolean validarCPF(long cpf){
		boolean valido = true;

		String validacaoCPF = Long.toString(cpf);

		int soma = 0, soma2 = 0, completaZeros = 11 - validacaoCPF.length();

		for(int i = 0; i < completaZeros; i++){
			validacaoCPF = "0" + validacaoCPF;
		}

		for(int i = 0; i < 9; i++){
			soma = soma + Integer.parseInt(Character.toString(validacaoCPF.charAt(i))) * (10 - i);
			soma2 = soma2 + Integer.parseInt(Character.toString(validacaoCPF.charAt(i))) * (11 - i);
		}
		soma2 = soma2 + Integer.parseInt(Character.toString(validacaoCPF.charAt(9))) * (2);

		soma = soma % 11;
		if(soma == 0 || soma == 1){
			soma = 0;
		}else{
			soma = 11 - soma;
		}

		if(soma != Integer.parseInt(Character.toString(validacaoCPF.charAt(9)))){
			valido = false;
		}else{
			soma2 = soma2 % 11;

			if(soma2 == 0 || soma2 == 1){
				soma2 = 0;
			}else{
				soma2 = 11 - soma2;
			}

			if(soma2 != Integer.parseInt(Character.toString(validacaoCPF.charAt(10)))){
				valido = false;
			}
		}

		return valido;
	}

	public boolean validarSenha(String senha){
		return this.senha.equals(senha);
	}

	public long getCpf() {
		return this.cpf;
	}

	public void setCpf(long cpf) throws CpfInvalidoException {
		if(Usuario.validarCPF(cpf)){
			this.cpf = cpf;
		}else{
			throw new CpfInvalidoException();
		}
	}

	public String getSenha() {
		return this.senha;
	}

	public void setSenha(String senha) throws SenhaInvalidaException {
		if(Usuario.senhaValida(senha)){
			this.senha = senha;
		}else{
			throw new SenhaInvalidaException();
		}
	}

	public String getNome() {
		return this.nome;
	}

	public void setNome(String nome) throws NomeInvalidoException {
		if(Usuario.nomeValido(nome)){
			this.nome = nome;
		}else{
			throw new NomeInvalidoException();
		}
	}

	public String getEmail() {
		return this.email;
	}

	public void setEmail(String email) throws EmailInvalidoException {
		if(Usuario.emailValido(email)){
			this.email = email;
		}else{
			throw new EmailInvalidoException();
		}
	}

	@Override
	public boolean equals(Object o) {
		return o instanceof Usuario && this.cpf == ((Usuario)o).getCpf();
	}

	@Override
	public String toString() {
		return String.format("CPF: %d    Nome: %s", this.cpf, this.nome);
	}

	public abstract boolean temPermissaoCadastro(Usuario user);

	public abstract boolean temPermissaoRemocao(Usuario user);

	public abstract boolean temPermissaoAlteracao(Usuario user);

}
