package servicos;

import java.rmi.Remote;
import java.rmi.RemoteException;
import util.IteratorBidirecional;

import negocio.exception.AvaliacaoExistenteException;
import negocio.exception.AvaliacaoInexistenteException;
import negocio.exception.ClassificacaoExistenteException;
import negocio.exception.ClassificacaoInexistenteException;
import negocio.exception.CopiaExistenteException;
import negocio.exception.CopiaInexistenteException;
import negocio.exception.CopiaLocadaNoPeriodoException;
import negocio.exception.CopiaReservadaNoPeriodoException;
import negocio.exception.FilialExistenteException;
import negocio.exception.FilialInexistenteException;
import negocio.exception.FilialNaoLogadaException;
import negocio.exception.LinkFotoExistenteException;
import negocio.exception.LocacaoExistenteException;
import negocio.exception.PlanoExistenteException;
import negocio.exception.PlanoInexistenteException;
import negocio.exception.ProdutoExistenteException;
import negocio.exception.ProdutoInexistenteException;
import negocio.exception.PromocaoExistenteException;
import negocio.exception.PromocaoInexistenteException;
import negocio.exception.UsuarioExistenteException;
import negocio.exception.UsuarioNaoLogadoException;
import negocio.exception.UsuarioSemPermissaoException;

import dados.exception.AvaliacaoInvalidaException;
import dados.exception.EmailInvalidoException;
import dados.exception.FilialInvalidaException;
import dados.exception.LocacaoInvalidaException;
import dados.exception.PlanoInvalidoException;
import dados.exception.PrecoInvalidoException;
import dados.exception.ProdutoInvalidoException;
import dados.exception.PromocaoInvalidaException;
import dados.filial.Endereco;
import dados.filial.Filial;
import dados.locacao.Copia;
import dados.locacao.Locacao;
import dados.locacao.Periodo;
import dados.locacao.Promocao;
import dados.produto.AvaliacaoProduto;
import dados.produto.ClassificacaoPreco;
import dados.produto.LinkFoto;
import dados.produto.Produto;
import dados.usuario.Cliente;
import dados.usuario.Plano;
import dados.usuario.Telefone;
import dados.usuario.Usuario;

/**
 * Esta classe representa a interface de comunicao do Servidor Web com o Servidor de Banco de Dados.
 * Ser utilizado o RMI para a comunicao entre ambos os servidores.
 * @author Caio Csar Sabino Silva
 */
public interface InterfaceWebBD extends Remote, InterfaceGeral {
	
	public abstract void cadastrarCliente(Cliente cliente, Telefone[] tarray) throws RemoteException, UsuarioExistenteException, EmailInvalidoException;

	public abstract void cadastrarFilial(long sessao, Filial filial) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialExistenteException, FilialInvalidaException;
	
	public abstract Filial buscarFilial(String id) throws RemoteException, FilialInexistenteException;
	
	public abstract void atualizarFilial(long sessao, Filial dados) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialInexistenteException, FilialInvalidaException;
	
	public abstract void descadastrarFilial(long sessao, String id) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, FilialInexistenteException;
	
	public abstract IteratorBidirecional<Filial> getFiliais() throws RemoteException, FilialInexistenteException;
	
	public abstract void cadastrarPlano(long sessao, Plano plano) throws RemoteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, PlanoInvalidoException, PlanoExistenteException;
	
	public abstract void atualizarPlano(long sessao, Plano atual) throws RemoteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, PlanoInexistenteException, PlanoInvalidoException;
	
	public abstract void descadastrarPlano(long sessao, String nome) throws RemoteException, PlanoInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException;
	
	public abstract void cadastrarPromocao(long sessao, Promocao promocao) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, PromocaoInvalidaException, PromocaoExistenteException;
	
	public abstract void mudarDescricaoPromocao(long sessao, Promocao promocao, String novaDescricao) throws RemoteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, PromocaoInexistenteException, PromocaoInvalidaException;
	
	public abstract void descadastrarPromocao(long sessao, String nome) throws RemoteException, UsuarioSemPermissaoException, UsuarioNaoLogadoException, PromocaoInexistenteException;
	
	/**
	 * Este mtodo cadastra um produto recebido como parmetro.
	 * @param sessao
	 * @param Produto a ser cadastrado
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioSemPermissaoException 
	 * @throws ProdutoInvalidoException 
	 * @throws ProdutoExistenteException 
	 * @throws RemoteException 
	 */
	public abstract void cadastrarProduto(long sessao, Produto produto) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, ProdutoExistenteException, ProdutoInvalidoException, RemoteException;
	
	public abstract void alterarDadosProduto(long sessao, Produto dados) throws UsuarioSemPermissaoException, UsuarioNaoLogadoException, ProdutoInexistenteException, ProdutoInvalidoException, RemoteException, ProdutoExistenteException;
	
	/**
	 * Este mtodo descadastra o produto de cdigo recebido como parmetro.
	 * @param sessao
	 * @param codigo do Produto a ser removido
	 * @throws ProdutoInexistenteException 
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioSemPermissaoException 
	 * @throws RemoteException 
	 */
	public abstract void descadastrarProduto(long sessao, String codigoProduto) throws ProdutoInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, RemoteException;
	
	/**
	 * Este mtodo cadastra uma cpia recebida a um produto fornecido.
	 * @param sessao
	 * @param copia a ser cadastrada
	 * @param produto da copia
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioSemPermissaoException 
	 * @throws ProdutoInexistenteException 
	 * @throws CopiaExistenteException 
	 * @throws RemoteException 
	 */
	public abstract void cadastrarCopia(long sessao, Copia copia) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, CopiaExistenteException, ProdutoInexistenteException, RemoteException;
	
	/**
	 * Este mtodo descadastra uma cpia com cdigo recebido
	 * @param sessao
	 * @param codigoCopia
	 * @throws UsuarioNaoLogadoException 
	 * @throws UsuarioSemPermissaoException 
	 * @throws ProdutoInexistenteException 
	 * @throws CopiaInexistenteException 
	 * @throws RemoteException 
	 */
	public abstract void descadastrarCopia(long sessao, String nomeProduto, int codigoCopia) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, ProdutoInexistenteException, CopiaInexistenteException, RemoteException;
	
	public abstract void avaliarProduto(long sessao, AvaliacaoProduto avaliacao) throws UsuarioNaoLogadoException, UsuarioSemPermissaoException, AvaliacaoExistenteException, AvaliacaoInvalidaException, RemoteException;
	
	public abstract AvaliacaoProduto buscarAvaliacaoProduto(long sessao, Produto produto) throws UsuarioNaoLogadoException, AvaliacaoInexistenteException, RemoteException;
	
	public abstract IteratorBidirecional<AvaliacaoProduto> getAvaliacaoesProduto(Produto produto) throws AvaliacaoInexistenteException, RemoteException;
	
	public abstract IteratorBidirecional<AvaliacaoProduto> getAvaliacaoesUsuario(long sessao) throws UsuarioNaoLogadoException, AvaliacaoInexistenteException, RemoteException;
	
	public abstract void removerAvaliacaoProduto(long sessao, Produto produto) throws UsuarioNaoLogadoException, AvaliacaoInexistenteException, RemoteException;
	
	public abstract void inserirLocacaoDomicilio(long sessao, Locacao locacao, Endereco endereco) throws LocacaoExistenteException, LocacaoInvalidaException, RemoteException, CopiaLocadaNoPeriodoException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, CopiaReservadaNoPeriodoException, FilialNaoLogadaException;
	
	public abstract void mudarFilialDeCopia(long sessao, Produto produto, Copia copia, Filial filial) throws RemoteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException, ProdutoInexistenteException, CopiaInexistenteException, FilialInvalidaException;
	
	public abstract void inserirClassificacao(long sessao, ClassificacaoPreco classificacao) throws RemoteException, UsuarioNaoLogadoException, ClassificacaoExistenteException, UsuarioSemPermissaoException;
	
	public abstract ClassificacaoPreco buscarClassificacao(String descricao) throws RemoteException, ClassificacaoInexistenteException;
	
	public abstract void removerClassificacao(long sessao, String descricao) throws RemoteException, ClassificacaoInexistenteException, UsuarioNaoLogadoException, UsuarioSemPermissaoException;
	
	public abstract void atualizar(long sessao, ClassificacaoPreco classificacao) throws RemoteException, ClassificacaoInexistenteException, UsuarioNaoLogadoException, PrecoInvalidoException, UsuarioSemPermissaoException;
	
	public abstract IteratorBidirecional<Filial> getFiliaisDisponiveis(Usuario user, Produto produto, Periodo p) throws FilialInexistenteException, ProdutoInexistenteException, RemoteException;
	
	public abstract IteratorBidirecional<Produto> getProdutosRecomendados() throws RemoteException, ProdutoInexistenteException;

	public abstract IteratorBidirecional<ClassificacaoPreco> getClassificacoes() throws RemoteException, ClassificacaoInexistenteException;
	
	public abstract void inserirLinkFoto(String nome, LinkFoto foto) throws LinkFotoExistenteException, ProdutoInexistenteException, RemoteException;

	public abstract Filial buscarFilial(long id) throws FilialInexistenteException, RemoteException;

	public abstract ClassificacaoPreco buscarClassificacao(long id) throws ClassificacaoInexistenteException, RemoteException;
	
}
